@@ -508,11 +508,12 @@ from .dxfile_functions import open_dxfile, new_dxfile, download_dxfile, upload_l
 from .dxgtable import DXGTable, NULL, DXGTABLE_HTTP_THREADS
 from .dxgtable_functions import open_dxgtable, new_dxgtable
 from .dxrecord import DXRecord, new_dxrecord
-from .dxworkflow import DXWorkflow
 from .dxproject import DXContainer, DXProject
 from .dxjob import DXJob, new_dxjob
+from .dxanalysis import DXAnalysis
 from .dxapplet import DXExecutable, DXApplet
 from .dxapp import DXApp
+from .dxworkflow import DXRecordWorkflow, DXAnalysisWorkflow, DXWorkflow
 from .auth import user_info
 from .search import (find_data_objects, find_jobs, find_projects, find_apps, find_one_data_object, find_one_project,
                      find_one_app)
